/**
  *  \file game/movement.h
  *  \brief Whole-Universe Movement Prediction
  *
  *  (c) 2001-2021 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_MOVEMENT_H
#define PCC_V2_GAME_MOVEMENT_H

#include "game/coord.h"
#include "game/cost.h"
#include "game/cargotype.h"

class GUniverse;
class GShip;
class GShipTurnPredictor;

class GMovementPredictor {
 public:
    GMovementPredictor();
    ~GMovementPredictor();

    void computeMovement(GUniverse& univ);

    GPoint getShipPosition(int sid) const;
    GCost getShipCargo(int sid) const;

 private:
    enum Status {
        NonExisting,            // ship does not exist
        Normal,                 // ship moves normally
        Towing,                 // ship is towing another one
        Towed,                  // ship is being towed; do not move it
        ResolvingLoop,          // temporary state for resolving intercept loops
        Moved                   // ship has moved
    };
    struct Info {
        Status status : 8;
        GPoint pos;             // if Moved, current position. Otherwise: waypoint.
        GCost cargo;

        Info()
            : status(NonExisting),
              pos(0, 0),
              cargo()
            { }
    };
    typedef std::vector<Info> InfoVector;
    InfoVector info;

    void init(const GUniverse& univ);
    void resolveTows(const GUniverse& univ);
    bool moveShips(const GUniverse& univ);
    void writeBack(GUniverse& univ);

    static bool isValidIntercept(const GShip& sh, const InfoVector& info);
    static void copyCargo(Info& info, const GShip& sh, GCargoType el);
    static void copyCargo(Info& info, const GShipTurnPredictor& pred);
};

#endif
