/**
  *  \file game/proxymbox.cc
  *
  *  (c) 2001-2021 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "game/proxymbox.h"

GProxyMailbox::GProxyMailbox(GMessagebox& outer, const std::vector<int>& indexes)
    : outer(outer), indexes(indexes)
{ }

int
GProxyMailbox::getCount()
{
    return static_cast<int>(indexes.size());
}

string_t
GProxyMailbox::getText(int index)
{
    int mapped = mapIndex(index);
    if (mapped >= 0 && mapped < outer.getCount()) {
        return outer.getText(mapped);
    } else {
        return string_t();
    }
}

string_t
GProxyMailbox::getHeading(int index)
{
    int mapped = mapIndex(index);
    if (mapped >= 0 && mapped < outer.getCount()) {
        return outer.getHeading(mapped);
    } else {
        return string_t();
    }
}

void
GProxyMailbox::importOuterCursor()
{
    for (size_t i = 0, n = indexes.size(); i < n; ++i) {
        if (indexes[i] == outer.getCurrent()) {
            setCurrent(int(i));
            break;
        }
    }
}

void
GProxyMailbox::exportOuterCursor()
{
    int mapped = mapIndex(getCurrent());
    if (mapped >= 0 && mapped < outer.getCount()) {
        outer.setCurrent(mapped);
    }
}

int
GProxyMailbox::mapIndex(int n)
{
    if (n >= 0 && n < int(indexes.size())) {
        return indexes[n];
    } else {
        return -1;
    }
}
